/*
 * $XFree86: $
 *
 * Copyright  2002 Carl D. Worth
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of Carl
 * D. Worth not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior
 * permission.  Carl D. Worth makes no representations about the
 * suitability of this software for any purpose.  It is provided "as
 * is" without express or implied warranty.
 *
 * CARL D. WORTH DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL CARL D. WORTH BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl Worth, USC/Information Sciences Institute */

#ifndef _XC_H_
#define _XC_H_

#include <X11/extensions/Xrender.h>
#include <X11/Xfuncproto.h>
#include <fontconfig/fontconfig.h>
#include <ic.h>
#include <pixregion.h>

typedef struct _XcSurface		XcSurface;
typedef XRenderColor			XcColor;

_XFUNCPROTOBEGIN

/* surface creation/destruction */

/* XXX: What's the best way to support both windows and pixmaps? Right
   now, this function uses exclusively visual or render_format. Visual
   is examined first and used if non-null. */
XcSurface *
XcSurfaceCreateForDrawable (Display	*dpy,
			    Drawable	drawable, 
			    Visual	*visual,
			    int		render_format,
			    Colormap	colormap);

XcSurface *
XcSurfaceCreateForIcImage (IcImage	*image);

void
XcSurfaceDestroy (XcSurface	*surface);

/* surface manipulation */

/* XXX: Need to resolve Region vs. PixRegion * before this can work
void
XcSurfaceSetClipRegion (XcSurface	*surface,
			Region		region);
*/

void
XcSurfaceSetAlphaMap (XcSurface	*surface,
		      XcSurface	*alpha);

void
XcSurfaceSetTransform (XcSurface	*surface,
		       XTransform	*transform);

void
XcSurfaceGetTransform (XcSurface	*surface,
		       XTransform	*transform);

typedef enum {
    XcFilterFast = IcFilterFast,
    XcFilterGood = IcFilterGood,
    XcFilterBest = IcFilterBest,
    XcFilterNearest = IcFilterNearest,
    XcFilterBilinear = IcFilterBilinear
} XcFilter;

void
XcSurfaceSetFilter (XcSurface	*surface,
		    XcFilter	filter);

void
XcSurfaceSetRepeat (XcSurface	*surface,
		    int		repeat);

/* XXX: Questions: should the data pointer be unsigned? Do we want to
   switch away from the char * for reasons of endian-ness? */
void
XcSurfacePutImage (XcSurface		*surface,
		   unsigned char	*data,
		   unsigned int		width,
		   unsigned int		height,
		   unsigned int		stride);

/* compositing */

void
XcComposite (int		op,
	     XcSurface		*src,
	     XcSurface		*mask,
	     XcSurface		*dst,
	     int		src_x,
	     int		src_y,
	     int		mask_x,
	     int		mask_y,
	     int		dst_x,
	     int		dst_y,
	     unsigned int	width,
	     unsigned int	height);

void
XcFillRectangle (int		op,
		 XcSurface	*dst,
		 const XcColor	*color,
		 int		x,
		 int		y,
		 unsigned int	width,
		 unsigned int	height);

void
XcFillRectangles (int			op,
		  XcSurface		*dst,
		  const XcColor		*color,
		  const XRectangle	*rects,
		  int			nrects);

void
XcCompositeTrapezoids (int		op,
		       XcSurface	*src,
		       XcSurface	*dst,
		       int		xSrc,
		       int		ySrc,
		       const XTrapezoid	*traps,
		       int		ntraps);

void
XcCompositeTriangles (int		op,
		      XcSurface		*src,
		      XcSurface		*dst,
		      int		xSrc,
		      int		ySrc,
		      const XTriangle	*tris,
		      int		ntris);

/* XXX: This function needs to disappear at some point. Xc is intended
   to entirely abstract away whether Render is present or not, hence
   it shouldn't go exposing Render data structures to the outside
   world. For now, though, while Xr is using Xft and until we decide
   the long-term plan for sharing code between Xft/Xc, Xr needs this
   function. */
Picture
XcSurfaceGetPicture(XcSurface *surface);

_XFUNCPROTOEND

#endif
