/*
 * $XFree86: $
 *
 * Copyright  2002 Carl D. Worth
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of Carl
 * D. Worth not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior
 * permission.  Carl D. Worth makes no representations about the
 * suitability of this software for any purpose.  It is provided "as
 * is" without express or implied warranty.
 *
 * CARL D. WORTH DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL CARL D. WORTH BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl Worth, USC/Information Sciences Institute */

#include "xcint.h"

void
XcComposite (int		op,
	     XcSurface		*src,
	     XcSurface		*mask,
	     XcSurface		*dst,
	     int		src_x,
	     int		src_y,
	     int		mask_x,
	     int		mask_y,
	     int		dst_x,
	     int		dst_y,
	     unsigned int	width,
	     unsigned int	height)
{
    if (dst->type == XC_SURFACE_TYPE_DRAWABLE
	&& XC_SURFACE_RENDER_HAS_COMPOSITE (dst)
	&& src->dpy == dst->dpy
	&& (mask == NULL || mask->dpy == dst->dpy)) {

	XRenderComposite (dst->dpy, op,
			  src->picture,
			  mask ? mask->picture : 0,
			  dst->picture,
			  src_x, src_y,
			  mask_x, mask_y,
			  dst_x, dst_y,
			  width, height);
    } else {
	_XcSurfacePullImage (src);
	_XcSurfacePullImage (mask);
	_XcSurfacePullImage (dst);

	IcComposite (op,
		     src->icimage,
		     mask ? mask->icimage : NULL,
		     dst->icimage,
		     src_x, src_y,
		     mask_x, mask_y,
		     dst_x, dst_y,
		     width, height);

	_XcSurfacePushImage (dst);
    }

}

void
XcFillRectangle (int		op,
		 XcSurface	*dst,
		 const XcColor	*color,
		 int		x,
		 int		y,
		 unsigned int	width,
		 unsigned int	height)
{
    if (dst->type == XC_SURFACE_TYPE_DRAWABLE
	&& XC_SURFACE_RENDER_HAS_FILL_RECTANGLE (dst)) {

	XRenderFillRectangle (dst->dpy, op, dst->picture,
			      color,
			      x, y, width, height);

    } else {
	_XcSurfacePullImage (dst);

	/* XXX: This cast should only occur with a #define hint from libic that it is OK */
	IcFillRectangle (op, dst->icimage,
			 (IcColor *) color,
			 x, y, width, height);

	_XcSurfacePushImage (dst);
    }
}

void
XcFillRectangles (int			op,
		  XcSurface		*dst,
		  const XcColor		*color,
		  const XRectangle	*rects,
		  int			nrects)
{
    if (dst->type == XC_SURFACE_TYPE_DRAWABLE
	&& XC_SURFACE_RENDER_HAS_FILL_RECTANGLE (dst)) {

	XRenderFillRectangles (dst->dpy, op, dst->picture,
			       color, rects, nrects);

    } else {
	_XcSurfacePullImage (dst);

	/* XXX: These casts should only occur with a #define hint from libic that it is OK */
	IcFillRectangles (op, dst->icimage,
			  (IcColor *) color, (IcRectangle *) rects, nrects);

	_XcSurfacePushImage (dst);
    }
}

void
XcCompositeTrapezoids (int		op,
		       XcSurface	*src,
		       XcSurface	*dst,
		       int		xSrc,
		       int		ySrc,
		       const XTrapezoid	*traps,
		       int		ntraps)
{
    if (dst->type == XC_SURFACE_TYPE_DRAWABLE
	&& XC_SURFACE_RENDER_HAS_TRAPEZOIDS (dst)
	&& src->dpy == dst->dpy) {

	XRenderCompositeTrapezoids (dst->dpy, op, src->picture, dst->picture,
				    XRenderFindStandardFormat (dst->dpy, PictStandardA8),
				    xSrc, ySrc, traps, ntraps);
    } else {
	_XcSurfacePullImage (src);
	_XcSurfacePullImage (dst);

	/* XXX: This cast should only occur with a #define hint from libic that it is OK */
	IcCompositeTrapezoids (op, src->icimage, dst->icimage,
			       xSrc, ySrc, (IcTrapezoid *) traps, ntraps);

	_XcSurfacePushImage (dst);
    }
}

void
XcCompositeTriangles (int		op,
		      XcSurface		*src,
		      XcSurface		*dst,
		      int		xSrc,
		      int		ySrc,
		      const XTriangle	*tris,
		      int		ntris)
{
    if (dst->type == XC_SURFACE_TYPE_DRAWABLE
	&& XC_SURFACE_RENDER_HAS_TRIANGLES (dst)
	&& src->dpy == dst->dpy) {

	XRenderCompositeTriangles (dst->dpy, op, src->picture, dst->picture,
				   XRenderFindStandardFormat (dst->dpy, PictStandardA8),
				   xSrc, ySrc, tris, ntris);
    } else {
	_XcSurfacePullImage (src);
	_XcSurfacePullImage (dst);

	/* XXX: This cast should only occur with a #define hint from libic that it is OK */
	IcCompositeTriangles (op, src->icimage, dst->icimage,
			      xSrc, ySrc, (IcTriangle *) tris, ntris);

	_XcSurfacePushImage (dst);
    }
}

Picture
XcSurfaceGetPicture (XcSurface *surface)
{
    return surface->picture;
}
