/*
 * $XFree86: $
 *
 * Copyright  2002 Carl D. Worth
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that the above copyright notice appear in all copies
 * and that both that copyright notice and this permission notice
 * appear in supporting documentation, and that the name of Carl
 * D. Worth not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior
 * permission.  Carl D. Worth makes no representations about the
 * suitability of this software for any purpose.  It is provided "as
 * is" without express or implied warranty.
 *
 * CARL D. WORTH DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL CARL D. WORTH BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * Author: Carl Worth, USC/Information Sciences Institute */

#ifndef _XCINT_H_
#define _XCINT_H_

#include "Xc.h"

#define XC_SURFACE_RENDER_AT_LEAST(surface, major, minor) \
	(((surface)->render_major > major) ? 1		  \
	 : ((surface)->render_major == major) ? ((surface)->render_minor >= minor) : 0)

#define XC_SURFACE_RENDER_HAS_CREATE_PICTURE(surface)		XC_SURFACE_RENDER_AT_LEAST((surface), 0, 0)
#define XC_SURFACE_RENDER_HAS_COMPOSITE(surface)		XC_SURFACE_RENDER_AT_LEAST((surface), 0, 0)

#define XC_SURFACE_RENDER_HAS_FILL_RECTANGLE(surface)		XC_SURFACE_RENDER_AT_LEAST((surface), 0, 1)
#define XC_SURFACE_RENDER_HAS_FILL_RECTANGLES(surface)		XC_SURFACE_RENDER_AT_LEAST((surface), 0, 1)

#define XC_SURFACE_RENDER_HAS_DISJOINT(surface)			XC_SURFACE_RENDER_AT_LEAST((surface), 0, 2)
#define XC_SURFACE_RENDER_HAS_CONJOINT(surface)			XC_SURFACE_RENDER_AT_LEAST((surface), 0, 2)

#define XC_SURFACE_RENDER_HAS_TRAPEZOIDS(surface)		XC_SURFACE_RENDER_AT_LEAST((surface), 0, 4)
#define XC_SURFACE_RENDER_HAS_TRIANGLES(surface)		XC_SURFACE_RENDER_AT_LEAST((surface), 0, 4)
#define XC_SURFACE_RENDER_HAS_TRISTRIP(surface)			XC_SURFACE_RENDER_AT_LEAST((surface), 0, 4)
#define XC_SURFACE_RENDER_HAS_TRIFAN(surface)			XC_SURFACE_RENDER_AT_LEAST((surface), 0, 4)

#define XC_SURFACE_RENDER_HAS_PICTURE_TRANSFORM(surface)	XC_SURFACE_RENDER_AT_LEAST((surface), 0, 6)

typedef enum { XC_SURFACE_TYPE_DRAWABLE, XC_SURFACE_TYPE_ICIMAGE } XcSurfaceType;

struct _XcSurface {
    /* For any type */
    XcSurfaceType type;
    XTransform transform;

    /* For TYPE_DRAWABLE */
    Display *dpy;
    GC gc;
    Drawable drawable;
    Visual *visual;

    int render_major;
    int render_minor;

    Picture picture;

    /* For TYPE_ICIMAGE */
    IcImage *icimage;
};

/* xcsurface.c */

/* XXX: Can someone think of better names for these? These functions
   are used when there are two copies of the image data. One where the
   user wants it, and another one local for manipulation. */
void
_XcSurfacePullImage (XcSurface *surface);

void
_XcSurfacePushImage (XcSurface *surface);

#endif
